// Marmalade headers
#include "s3e.h"

// iwGame engine headers

// Game headers
#include "Common.h"
#include "Game.h"
#include "IwGameHttp.h"
#include "IwGameFile.h"
#include "IwGameAdsView.h"
#include "IwGameAdsViewAnimator.h"

void AdTest_Init()
{
	int width = IwGxGetScreenWidth();
	int height = IwGxGetScreenHeight();

	// Create ad view
	CIwGameAdsView::Create();

	// INitialise with Application ID (yuo get this from your ad provider)
	IW_GAME_ADS_VIEW->Init("");

	// Set ad provider
	IW_GAME_ADS_VIEW->setAdProvider(CIwGameAds::InnerActive);

	// Set ad request interval in seconds
	IW_GAME_ADS_VIEW->setNewAdInterval(30);

	// Force a request for an initial ad
	IW_GAME_ADS_VIEW->RequestNewAd(CIwGameAds::InnerActive);

	// Set total number of ads visible in te ads view
	IW_GAME_ADS_VIEW->setNumAdsVisible(1);

	// Tell animators to loop
	IW_GAME_ADS_VIEW->setLooped(true);

	// Create and attach an animator that fades the ad in over 1 second, pauses for 7 seconds and then fades the ad back out
	CIwGameAdsViewAnimator* anim = new CIwGameAdsViewAnimator();
	anim->Init();
	anim->setAdViewDataIndex(0);
	anim->setCanvasSize(width, height);
	anim->setInAnim(CIwGameAdsViewAnimator::AnimFadeIn, 1000);
	anim->setOutAnim(CIwGameAdsViewAnimator::AnimFadeOut, 1000);
	anim->setStayDuration(7000);
	IW_GAME_ADS_VIEW->addAnimator(0, anim);

	// Create and attach an animator that sweeps the ad in from the right the over 1,2 seconds, pauses for 7 seconds and then sweeps back out
	anim = new CIwGameAdsViewAnimator();
	anim->Init();
	anim->setAdViewDataIndex(0);
	anim->setCanvasSize(width, height);
	anim->setRestingPosition(0, -height / 8);
	anim->setInAnim(CIwGameAdsViewAnimator::AnimRightSweepIn, 1200);
	anim->setOutAnim(CIwGameAdsViewAnimator::AnimRightSweepOut, 1200);
	anim->setStayDuration(7000);
	IW_GAME_ADS_VIEW->addAnimator(0, anim);

	// Create and attach an animator that scales the ad in over 1.5 seconds, pauses for 7 seconds and then scales back out
	anim = new CIwGameAdsViewAnimator();
	anim->Init();
	anim->setAdViewDataIndex(0);
	anim->setCanvasSize(width, height);
	anim->setInAnim(CIwGameAdsViewAnimator::AnimScaleIn, 1500);
	anim->setOutAnim(CIwGameAdsViewAnimator::AnimScaleOut, 1500);
	anim->setStayDuration(7000);
	IW_GAME_ADS_VIEW->addAnimator(0, anim);

	// Create and attach an animator that rotates the ad in over 1 second, pauses for 7 seconds and then rotates back out
	anim = new CIwGameAdsViewAnimator();
	anim->Init();
	anim->setAdViewDataIndex(0);
	anim->setCanvasSize(width, height);
	anim->setInAnim(CIwGameAdsViewAnimator::AnimSpinIn, 1000);
	anim->setOutAnim(CIwGameAdsViewAnimator::AnimSpinOut, 1000);
	anim->setStayDuration(7000);
	IW_GAME_ADS_VIEW->addAnimator(0, anim);
}

void AdTest_Release()
{
	IW_GAME_ADS_VIEW->Release();
	CIwGameAdsView::Destroy();
}

void AdTest_Update()
{
	// Update the ads view
	IW_GAME_ADS_VIEW->Update(GAME->getCurrentScene(), 1.0f);
	IW_GAME_ADS_VIEW->Draw(GAME->getCurrentScene());
}

int main()
{
	// Init Game
	Game::Create();
	GAME->Init();

	// Initialise the http manager
	CIwGameHttpManager::Create();
	IW_GAME_HTTP_MANAGER->Init();

	// Download an image file from the web
	CIwGameFile* image_file = new CIwGameFile();
	image_file->Open("http://www.battleballz.com/bb_icon.gif", false, true);
	image_file->Close();

	// Create an image from the downloaded gif file
	CIwGameImage* image = new CIwGameImage();
	if (image_file->isFileAvailable() && image_file->getError() == CIwGameFile::ErrorNone)
		image->Init("", image_file->getContent(), image_file->getContentLength(), false);
	delete image_file;

	// Create a test sprite to display our downloaded gif
	CIwGameBitmapSprite* test_sprite = new CIwGameBitmapSprite();
	test_sprite->setImage(image);
	test_sprite->setDestSize(image->getWidth(), image->getHeight());
	test_sprite->setSrcRect(0, 0, image->getWidth(), image->getHeight());
	test_sprite->setPosition(20 + image->getWidth() / 2, 20 + image->getHeight() / 2);

	// Init Ad Test
	AdTest_Init();
	
	// Main Game Loop
	while (!s3eDeviceCheckQuitRequest())
	{
		// Update the game
		if (!GAME->Update())
			break;

		// Check for back button quit
		if (IW_GAME_INPUT->isKeyDown(s3eKeyAbsBSK))
			break;

		// Draw the scene
		GAME->Draw();

		// Draw test sprite to show remote image downloading working
		test_sprite->Draw();

		// Update the Ad test
		AdTest_Update();

		// Update http manager
		IW_GAME_HTTP_MANAGER->Update();
	}

	// Clean up image
	if (image != NULL)
		delete image;
	if (test_sprite != NULL)
		delete test_sprite;

	// Release Ad Test
	AdTest_Release();

	// Clean up http manager
	IW_GAME_HTTP_MANAGER->Release();
	CIwGameHttpManager::Destroy();

	GAME->Release();
	Game::Destroy();

    return 0;
}

